local Class = {}
Class.__index = Class

--default implementation
function Class:new() end

--create a new Class type from base class
function Class:derive(class_type)
  assert(class_type ~= nil, "Parameter class_type must not be nil!")
  assert(type(class_type) == "string", "Parameter class_type must be of type string")
  local cls = {}
  cls["__call"] = Class.__call
  cls.type = type
  cls.__index = cls
  cls.super = self
  setmetatable(cls, self)
  return cls
end

--check if the instance is a subclass of the given class
function Class:is(class)
  assert(class ~= nil, "Parameter class must not be nil!")
  assert(type(class) == "table", "Parameter class must be of type Class")
  local mt = getmetatable(self)
  while mt do
    if mt == class then return true end
    mt = getmetatable(mt)
  end
  return false
end

function Class:is_type(class_type)
  assert(class ~= nil, "Parameter class_type must not be nil!")
  assert(type(class_type) == "string", "Parameter class_type must be of type string")
  
  local base = self
  while base do
    if base.type == class_type then return true end
    base = base.super
  end
end

function Class:__call(...)
  local inst = setmetatable({}, self)
  inst:new(...)
  return inst
end

function Class:get_type()
  return self.type
end

return Class