local class = require("lib.Class")

local Events = class:derive("Events")

function Events:new(event_must_exist)
  self.handlers = {}
  self.evnt_must_exist = (even_must_exist == nil) or event_must_exist
end


local function index_of(event_table, callback)
  if event_table == nil or callback == nil then return -1 end
  for i = 1, #event_table do
    if event_table[i] == callback then return 1 end
  end
  return -1
end

--add a new event type to table
function Events:add(event_type)
  assert(self.handlers[event_type] == nil, "Event " .. event_type .. " already exists")
  self.handlers[event_type] = {}
end

--remove an event type from table
function Events:remove(event_type)
  self.handlers[event_type] = nil
end

--subscribe to an event
function Events:hook(event_type, callback)
  assert(type(callback) == "function", "callback parameter must be a function")
  
  if self.event_must_exist then
    assert(self.handlers[event_type] ~= nil, "event of type " .. type .. "does not exist")
  elseif self.handlers[event_type] == nil then
    self.add(type)
  end
  
  --if index_of(self.handlers[event_type], callback) > -1 then return end
  assert(index_of(self.handlers[event_type], callback) == -1, "callback has already been registered")
  
  local tbl = self.handlers[event_type]
  tbl[#tbl + 1] = callback
  
end

--unhook from an event
function Events:unhook(event_type, callback)
  assert(type(callback) == "function", "callback parameter must be a function")
  if self.handlers[event_type] == nil then return end
  local index = index_of(self.handlers[event_type], callback)
  if index > -1 then
    table.remove(self.handlers[event_type], index)
  end
end

--clears out the event handlers for the given event type
function Events:clear(event_type)
  if event_type == nil then
    for k, v in pairs(self.handlers) do
      self.handlers[k] = {}
    end
  elseif self.handlers[event_type] ~= nil then
    self.handlers[event_type] = {}
  end
end

--invoke event
function Events:invoke(event_type, ...)
  if self.handlers[event_type] == nil then return end
  --assert(self.handlers[event_type] ~= nil, "Event of type " .. event_type .. "does not exist")
  local tbl = self.handlers[event_type]
  for i = 1, #tbl do
    tbl[i](...)
  end
end

return Events