local Class = require("lib.class")

local GPM = Class:derive("GamepadManager")

local DEAD_ZONE = 0.1

local function hook_love_events(self)
  
  function love.joystickadded(joystick)
    print(joystick:getID() .. " was just added as a joystick.")
    local id = joystick:getID()
    assert(self.connected_sticks[id] == nil or self.connected_sticks[id] == false, "Joystick " .. id .." already exists")
    self.connected_sticks[id] = joystick
    --print("added joystick " .. tostring(id))
    self.is_connected[id] = true
    --assert(self.is_connected[joyID] == true, "Joystick " .. joyID .. "was not connected properly.")
    --print(tostring(self.is_connected[id]))
    self.button_map[id] = {}
    --todo: invoke a controller_added event
    
  end
  
  function love.joystickremoved(joystick)
    local id = joystick:getID()
    self.is_connected[id] = false
    self.connected_sticks[id] = false
    self.button_map[id] = {}
    --todo: controller_removed event
  end

  function love.gamepadpressed(joystick, button)
    local id = joystick:getID()
    self.button_map[id][button] = true
  end
  
  function love.gamepadreleased(joystick, button)
    local id = joystick:getID()
    self.button_map[id][button] = false
  end
  
end

function GPM:new(db_files, ad_enabled)
  if #db_files ~= nil then
    for i = 1, #db_files do
      love.joystick.loadGamepadMappings(db_files[i])
    end
  end
  
  hook_love_events(self)
  
  --if true then left analog joystick will be converted to a dpad
  self.ad_enabled = ad_enabled
  
  --currently connected joysticks
  self.connected_sticks = {}
  self.is_connected = {}
  print(type(self.is_connected))
  --maps a joystick id to a table of key values where the key is a button and the values are:
  -- true == just_pressed, false == just_released, nil == none
  self.button_map = {}
  
  
end

function GPM:exists(joyID)
  return self.is_connected[joyID] ~= nil and self.is_connected[joyID]
end

--returns true if button on given joystick was pressed this frame
function GPM:button_down(joyID, button)
  print(type(self.is_connected) .. " in bdown")
  --print(self.is_connected[joyID])
  if self.is_connected[joyID] == nil or self.is_connected[joyID] == false then
    return false
  else return self.button_map[joyID][button] == true end
  
end

--returns true if button on given joystick was released this frame
function GPM:button_up(joyID, button)
  if self.is_connected[joyID] == nil or self.is_connected[joyID] == false then
    return false
  else return self.button_map[joyID][button] == false end
end

--return curr state of button for given joystick
function GPM:button(joyID, button)
  local stick = self.connected_sticks[joyID]
  if self.is_connected[joyID] == nil or self.is_connected[joyID] == false then
    return false
  end
  
  local is_down = stick:isGamepadDown(button)
  
  --do we want to conver the left analog stick to dpad
  if self.ad_enabled and not is_down then
    local xAxis = stick:getGamepadAxis("lefx")
    local yAxis = stick:getGamepadAxis("lefty")
    if button == "dpright" then
      is_down = xAxis > DEAD_ZONE
    elseif button == "dpleft" then
      is_down = xAxis < -DEAD_ZONE
    elseif button "dpup" then
      is_down = yAxis < -DEAD_ZONE
    elseif button == "dpdown" then
      is_down = yAxis > DEAD_ZONE
    end
  end
  return is_down
end

function GPM:update(dt)
  for i = 1, #self.connected_sticks do
    if self.button_map[i] then
      for k,_ in pairs(self.button_map[i]) do
        self.button_map[i][k] = nil
      end
    end
  end
end

function GPM:get_is_connected()
  return self.is_connected
end

















return GPM